#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "../plugindir/checksum.h"

static void printhelp(void);

int main(int argc, char** argv) {
  int i,res;
  char time[256];
  char cbuf[33];
  char oldchecksum[256];
  char dbname[256];
  int oldlen, len;
  int th, tm, ts, tcs, tms;
  FILE *dbfile;
  for(i=1; i<argc; i++) {
    if(strcasecmp(argv[i],"--help")==0 || strcasecmp(argv[i],"-h")==0) {
      printhelp();
      return 0;
    }
  }
  if(argc<3) {
    fprintf(stderr, "not enough parameters\n\n");
    printhelp();
    return -1;
  }
  res = filechecksum(cbuf, argv[1]);
  if(!res) {
    fprintf(stderr, "couldn't find file %s\n", argv[1]);
    return -1;
  }
  cbuf[32] = 0;

  strcpy(time, argv[2]);
  for(i=3; i<argc; i++) {
    strcat(time, " ");
    strcat(time, argv[i]);
  }

  th = tm = ts = tcs = tms = 0;
  res = sscanf(time, "%dmin%dsec%dms", &tm, &ts, &tms);
  if(res==3) goto gottime;
  res = sscanf(time, "%dmin%ds%dms", &tm, &ts, &tms);
  if(res==3) goto gottime;
  res = sscanf(time, "%dm%dsec%dms", &tm, &ts, &tms);
  if(res==3) goto gottime;
  res = sscanf(time, "%dm%ds%dms", &tm, &ts, &tms);
  if(res==3) goto gottime;
  res = sscanf(time, "%d:%d.%d", &tm, &ts, &tcs);
  if(res==3) goto gottime;
  res = sscanf(time, "%dm%ds", &tm, &ts);
  if(res==2) goto gottime;

  fprintf(stderr, "illegal length given: few examples of valid lengths:\nexample time 1: 6m5s (6 minutes 5 seconds)\nexample time 2: 1m5s100ms (1 minute 5 seconds 100 milliseconds)\nexample time 3: 3:46.42 (3 minutes 46 seconds 420 milliseconds given by unix time command)\n");
  return -1;

 gottime:
  len = tms + 10 * tcs + 1000 * ts + 1000*60*tm + 1000*60*60*th;
  //  fprintf(stderr, "%s %s %d\n", argv[1], cbuf, len);

  sprintf(dbname,"%s/.xmms/uade/db-content", getenv("HOME"));
  dbfile = fopen(dbname, "r");
  if(!dbfile) {
    fprintf(stderr, "dbfile %s does not exist...\nif you know what you are doing you can create this dbfile by one of two methods:\n (1) either copying file contrib/default-db/cb-content to $HOME/.xmms/uade/\n (2) by creating a blank dbfile with 'touch %s'\n", dbname, dbname);
    return -1;
  }
  while(1) {
    res = fscanf(dbfile, "%s %d\n", oldchecksum, &oldlen);
    if(res < 2) {
      break;
    }
    if(!strcasecmp(oldchecksum, cbuf)) {
      fprintf(stderr, "checksum %s already exists in the database (length = %d ms)\n => ABORTING\n", cbuf, oldlen);
      fprintf(stderr, "use 'grep -v %s %s > tmpfile; cp tmpfile %s' to remove the old entry\n", cbuf, dbname, dbname);
      return -1;
    }
  }
  fclose(dbfile);

  dbfile = fopen(dbname, "a");
  if(!dbfile) {
    fprintf(stderr, "could not append to dbfile %s\n", dbname);
    return -1;
  }
  fprintf(dbfile, "%s +%d\n", cbuf, len);
  fprintf(stderr, "added file %s to database with playtime %d milliseconds\n", argv[1], len);
  fclose(dbfile);
  return 0;
}

static void printhelp(void) {
  fprintf(stderr, "UADE song length tool version 666\n\n");
  fprintf(stderr, "Usage: songlen FILE PLAYTIME\n\n");
  fprintf(stderr, "Brief description: Adds file FILE to playtime database ($HOME/.xmms/uade/db-content) with a playtime PLAYTIME\nDo not use this tool unless you really know what you are doing\n\n");
  fprintf(stderr, "Example: ./songlen ../cust/VisionFactory2.cust 0m52s\n");
}
